#!/bin/bash
#
# All Rights Reserved, Copyright (c) FUJITSU LIMITED 2012-2015
#
#	Configuration Tool for Oracle VM Server for SPARC
#
# Revision 2.3  2015/07/10
#
# Start_config.sh

VER="ver.2.4"
TOP_TITLE=" The Configuration Tool for Oracle VM Server for SPARC"

# ID check
if [ `id -u` != 0 ];then
	echo "ERROR: You must have root privileges to run this tool."
	echo ""
	exit 1
fi	

PATH=/usr/bin:/usr/sbin:/opt/SUNWldm/bin ; export PATH
DATE=`date '+%y%m%d%H%M%S'`; export DATE
LANG=C ; export LANG
LC_ALL=C ; export LC_ALL

dir=`( cd \`echo $0 |sed -e 's#[^/]*$##; s#^$#./#'\` ; pwd )`
script_dir="$dir/script";export script_dir
conf_dir="$dir/conf";export conf_dir
work_dir=/var/tmp/work_$$;export work_dir

. $script_dir/template_menu.sh
. $script_dir/common.sh

. $script_dir/pattern1_primary.sh
. $script_dir/pattern1_guest.sh
. $script_dir/pattern2_primary.sh
. $script_dir/pattern2_io.sh
. $script_dir/pattern3_primary.sh
. $script_dir/pattern3_io.sh
. $script_dir/pattern4_primary.sh
. $script_dir/pattern4_io1.sh
. $script_dir/pattern4_io2.sh
. $script_dir/pattern4_io3.sh
. $script_dir/pattern4_web.sh
. $script_dir/pattern4_ap.sh

. $script_dir/custom_primary.sh
. $script_dir/custom_guest.sh
. $script_dir/custom_io.sh
. $script_dir/custom_service.sh

. $script_dir/init_all.sh
#. $script_dir/init_one.sh

P_CONF=$work_dir/primary.conf;export P_CONF
G_CONF=$work_dir/guest.conf;export G_CONF
I_CONF=$work_dir/io.conf;export I_CONF

VNTSD="svc:/ldoms/vntsd:default"

delete()
{
	if [ -d $work_dir ];then
		rm -fr $work_dir
		echo ""
	fi
}

trap 'echo "[interrupted]"; delete ; exit 1' 1 2 3 15
stty erase 

service_check()
{
	LDMD_CHECK=`svcs -l svc:/ldoms/ldmd:default | sed -n 4p|awk '{print $2}'`
#	LDMD_CHECK=online # DEBUG
	if [ "$LDMD_CHECK" != "online" ];then
		echo "ERROR: The ldmd (Logical Domains Manager) service is not online."
		echo "ERROR: Please start the ldmd service with the following command."
		echo "svcadm enable svc:/ldoms/ldmd:default"
		echo ""
		exit 0
	fi
}

#ver1.1
delayed_check()
{
	DL_CHECK=`ldm ls -p primary | grep DOMAIN | awk -F"|" '{print $4}' | sed -e s/^flags=// | cut -c 3`
#	DL_CHECK="-" # DEBUG
	if [ "$DL_CHECK" = "d" ];then
		echo "------------------------------------------------------------------------------------"
		echo "ERROR: The primary domain is in process of a delayed reconfiguration."
		echo "ERROR: Please reboot the Operating System before you run this tool."
		echo "(command: /usr/sbin/shutdown -y -g0 -i6)"
		echo "------------------------------------------------------------------------------------"
		exit 0
	fi
}
#ver1.1

primary_check()
{
	CONFSTATE=`ldm list-config | grep factory-default | awk '{print $2}'`
#	CONFSTATE=current # DEBUG
	if [ ${CONFSTATE:-[]} = "[current]" ];then
		echo "---------------------------------------------------------------------------------------"
		echo "Please set the control domain before you configure the guest domain or a I/O domain."
		echo "---------------------------------------------------------------------------------------"
		echo "Please press the enter key. \c"
		read key
		continue
	fi
}

#ver1.1
primary_check2()
{
	CONFSTATE=`ldm list-config | grep factory-default | awk '{print $2}'`
#	CONFSTATE="[current]" # DEBUG
	if [ ${CONFSTATE:-[]} != "[current]" ];then
		echo ""
		echo "---------------------------------------------------------------------------------------"
		echo "Need to restore the \"factory-default\" configuration that is initial state before you"
		echo "set the primary domain by this tool."
		echo "---------------------------------------------------------------------------------------"
		echo "Please press the enter key. \c"
		read key
		continue
	fi
}

#ver2.0.1
hard_check()
{
	if [ "$MODEL" != "M10-4" -a "$MODEL" != "M10-4S" -a "$MODEL" != "T4-4" ];then
		echo ""
		echo "---------------------------------------------------------------------------------------"
		echo "ERROR: Your hard does not support this pattern."
		echo "---------------------------------------------------------------------------------------"
		echo "Please press the enter key. \c"
		read key
		continue
	fi
}

arch_ver()
{
	CPUARCH=`prtconf -b | grep ORCL | awk -F"," '{print $2}'`
	if [ "$CPUARCH" = "SPARC64-X" ];then
		THREAD=2
	elif [ "$CPUARCH" = "SPARC-T4-2" -o "$CPUARCH" = "SPARC-T4-4" ];then
		THREAD=8
	else
		echo "This tool is not supported in this environment."
		delete
		exit 0
	fi	
	MODEL=`prtconf -b | grep banner-name | awk '{print $3}'`
	LMVER=`pkginfo -l SUNWldm | grep VERSION | awk '{print $2}' | cut -c 1`
}

header()
{
	clear
	MSS=$1
	DAI=`echo ${#MSS}`
	MJS=`expr $DAI`
	print "======================================================================================= "
	printf "%s            " "$1"
	MSS=$2
	DAI=`echo ${#MSS}`
	LPS=`expr 71 - $MJS - $DAI`
	for i in `seq 1 $LPS`
	do
		printf " "
  done
	printf " [%s] \n" "$2"
	print "======================================================================================= \n"
}


menu()
{
	delete
	service_check
	delayed_check # ver1.1
	arch_ver
	mkdir $work_dir
	while :
	do
	header "$TOP_TITLE" $VER
	cat <<-EOF
	Menu

	 1. Create a logical domain configuration.
	 2. Initialize the settings of the logical domain configuration.


	q:quit
	---------------------------------------------------------------------------------------
	EOF
	echo "Please select (1-2,q) ==> \c"
	read i

	case $i in
		[1])	sub_menu1
			;;
#		[2])	sub_menu2
		[2])	init_all
			exit 0
			;;
		[q])	quit
			;;
	esac
	done
}

sub_menu1()
{
	while :
	do
	header "$TOP_TITLE" $VER
	cat <<-EOF
	Select the method to create a configuration file.

	 1. Use a template.
	 2. Create an original configuration.


	t:top menu  q:quit
	---------------------------------------------------------------------------------------
	EOF
	echo "Please select (1-2,t,q) ==> \c"
	read i

	case $i in
		[1])	template
			;;
		[2])	custom
			;;
		[t])	menu
			;;
		[q])	quit
			;;
	esac
	done
}

#sub_menu2()
#{
#	while :
#	do
#	clear
#
#	cat <<-EOF
#	=======================================================================================
#	 The Configuration Tool for Oracle VM Server for SPARC                       [ver.2.0]
#	=======================================================================================
#
#	Select the method to initialize the settings of the logical domain configuration.
#
#	 1. All domain's configurations.
#	 2. One of the configurations that you choose.
#	
#
#	t:top menu  q:quit
#	---------------------------------------------------------------------------------------
#	EOF
#	echo "Please select (1-2,t,q) ==> \c"
#	read i
#
#	case $i in
#		[1])	init_all
#			exit 0
#			;;
#		[2])	init_one
#			exit 0
#			;;
#		[t])	menu
#			;;
#		[q])	quit
#			;;
#	esac
#	done
#}

template()
{
	while :
	do
	header "$TOP_TITLE" "Template"
	cat <<-EOF
	Select a template.

	 1. Pattern 1 [Create primary domain and guest domain] (for SPARC M10-1)
	 2. Pattern 2 [Create primary domain and io domain] (for SPARC M10-1)
	 3. Pattern 3 [Create primary domain, three io domains] (for SPARC M10-4)
	 4. Pattern 4 [Create primary domain, two io domain and some guest domains] (for SPARC 
	  M10-4)

	t:top menu  q:quit
	---------------------------------------------------------------------------------------
	EOF

#	 5. Pattern 5 [primary/io domain * 2/io domain * 1/guest domain] (for SPARC M10-4)

	echo "Please select (1-4,t,q) ==> \c"
	read i

	case $i in
		[1])	pattern1_temp 
			exit 0
			;;
		[2])	pattern2_temp 
			exit 0
			;;
		[3])	hard_check
			pattern3_temp 
			exit 0
			;;
		[4])	hard_check
			pattern4_temp 
			exit 0
			;;
#		[5])	pattern5_temp 
#			exit 0
#			;;
		[t])	menu
			;;
		[q])	quit
			;;
	esac
	done
}

custom()
{
	while :
	do
	header "$TOP_TITLE" "Original"
	cat <<-EOF
	Select the type of domain that you wish to create.

	 1. Primary domain (Control domain)
	 2. Guest domain
	 3. I/O domain or Service domain

	t:top menu  q:quit
	---------------------------------------------------------------------------------------
	EOF
	echo "Please select (1-3,t,q) ==> \c"
	read i

	case $i in
		[1])	primary_check2
			primary_custom
			exit 0
			;;
		[2])	primary_check
			guest_custom
			exit 0
			;;
		[3])	primary_check
			io_custom
			exit 0
			;;
		[t])	menu
			;;
		[q])	quit
			;;
	esac
	done
}

quit()
{
	echo "Are you sure you want to quit? Your progress will not be saved.(y/[n]) \c"
	read YesNo
	if [ ${YesNo:-n} = y ];then
		delete
		exit 0
	fi
}

top()
{
	echo "Are you sure you want to go to the top? Your progress will not be saved.(y/[n]) \c"
	read YesNo
	if [ ${YesNo:-n} = y ];then
		menu
	fi
}

menu
