#/bin/bash
#
# All Rights Reserved, Copyright (c) FUJITSU LIMITED 2012-2014
#
#	Configuration Tool for Oracle VM Server for SPARC
#
# Revision 2.2  2014/10/02
#
# common.sh

############################################################################
# Print I/O
############################################################################
print_io()
{
echo "No. DeviceName Type Domain" | awk '{printf"%-1s%-4s%-16s%-6s%-10s\n"," ",$1,$2,$3,$4}'
echo "--- --------------- ----- ---------" | awk '{printf"%-1s%-4s%-16s%-6s%-10s\n"," ",$1,$2,$3,$4}'

ldm list-io -p | grep "type=BUS" > $work_dir/list_$$

for i in `zpool status rpool | grep 'c.*d0' | awk '{print $1}'`
do
	D=`prtconf -v -c /dev/dsk/$i | grep pci | grep value | awk -F"/" '{print $2}'`
	while read line
	do
		echo $line | grep $D > /dev/null 2>&1
		if [ $? = 0 ];then
			echo $line | grep boot-device > /dev/null 2>&1
			if [ $? != 0 ];then
				echo "$line|[boot-device]" >> $work_dir/dev_$$
			else
				echo $line >> $work_dir/dev_$$
			fi
		else
			echo $line >> $work_dir/dev_$$
		fi
	done < $work_dir/list_$$ 

	mv $work_dir/dev_$$ $work_dir/list_$$
done

COUNT0=0
while read line
do
	B=""
	C="(Reserved)"
	if [ -f $work_dir/io_resv.tmp ];then
		while read line2
		do
			A=(`echo $line2`)
			echo $line | grep ${A[0]} > /dev/null 2>&1
			if [ $? = 0 ];then
				B=${A[1]}
				break
			fi
		done < $work_dir/io_resv.tmp
	fi
	if [ -z $B ];then
		if [ $LMVER = 3 ];then NO=8
		elif [ $LMVER = 2 ];then NO=7 ;fi
		echo "$COUNT0$line" | sed -e 's/dev=//' -e 's/type=//' -e 's/domain=//' | awk -F"|" '{printf"%-1s%-4s%-16s%-6s%-10s%-10s\n"," ",$1,$2,$6,$5,$V}' V="$NO"
	else
		if [ $LMVER = 3 ];then NO1=8 ;NO2=9
		elif [ $LMVER = 2 ];then NO1=7 ;NO2=8 ;fi
		echo "$COUNT0$line|$B|$C" | sed -e 's/dev=//' -e 's/type=//' -e 's/domain=//' | awk -F"|" '{printf"%-1s%-4s%-16s%-6s%-10s%-10s\n"," ",$1,$2,$6,$V,$W}' V="$NO1" W="$NO2"
	fi
	COUNT0=`expr $COUNT0 + 1`
done < $work_dir/list_$$

}

############################################################################
# Print CPU
############################################################################
print_cpu()
{
if [ "$MODEL" = "T4-4" ];then
	NUM_of_CORES=`ldm list-devices -a core | sed -n 3,\\$p | sed -e \\$d | wc -l`
else
	NUM_of_CORES=`ldm list-permits | sed -n 4,\\$p | awk '{print $1}'`
fi
NUM_of_REM=$NUM_of_CORES
for i in `ldm ls -p | grep DOMAIN | sed -e 's/name=//' -e 's/ncpu=//' | awk -F"|" '{print $6/V}' V=$THREAD`
do
	NUM_of_REM=`expr $NUM_of_REM - $i`
done

if [ -f $work_dir/cpu_$$ ];then
	for j in `cat $work_dir/cpu_$$ | awk '{print $2}'`
	do
		NUM_of_REM=`expr $NUM_of_REM - $j`	
	done
fi
if [ $DKIND = "primary" ];then
	NUM_of_REM=$NUM_of_CORES
fi
if [ -n "$CPU_MAX" ];then
	if [ $NUM_of_REM -gt $CPU_MAX ];then
		NUM_of_REM=$CPU_MAX
	fi
fi
echo "Domain Cores" | awk '{printf"%-1s%-18s%-5s\n","",$1,$2}' > $work_dir/cpulist.tmp
echo "----------------- -----" | awk '{printf"%-1s%-18s%-5s\n","",$1,$2}' >> $work_dir/cpulist.tmp
ldm ls -p | grep DOMAIN | sed -e 's/name=//' -e 's/ncpu=//' | awk -F"|" '{printf"%-1s%-18s%5s\n","",$2,$6/V}' V=$THREAD >> $work_dir/cpulist.tmp
if [ -f $work_dir/cpu_$$ ];then
	cat $work_dir/cpu_$$ | awk '{printf"%-1s%-18s%5s%-1s%-15s\n","",$1,$2,"",$3}' >> $work_dir/cpulist.tmp
fi
echo "-----------------------" | awk '{printf"%-1s%-24s\n","",$1}' >> $work_dir/cpulist.tmp
echo "" >> $work_dir/cpulist.tmp
if [ $DKIND = "primary" ];then
	eval echo "The quantity of CPU cores that can be allocated to the $DKIND domain: $CPU_MIN to $NUM_of_REM" >> $work_dir/cpulist.tmp
else
	eval echo "The quantity of CPU cores that can be allocated to the $DKIND domain: $NUM_of_REM" >> $work_dir/cpulist.tmp
fi
if [ "$NUM_of_REM" -le 0 ];then
	echo "There is no CPU resource that can be allocated for the guest domain." >> $work_dir/cpulist.tmp
fi
echo "" >> $work_dir/cpulist.tmp
}

############################################################################
# Print MEMORY
############################################################################
print_memory()
{
if [ $DKIND = "primary" ];then
	A=`ldm list-devices -p -a mem | grep size | grep -v sys | awk -F"|" '{print $3}' | sed -e 's/size=//'`
else
	A=`ldm list-devices -p -a mem | grep size | grep -v primary | grep -v sys | awk -F"|" '{print $3}' | sed -e 's/size=//'`
fi
NUM=0
for i in $A
do
	NUM=`expr $NUM + $i`
done
MEM_PRI=`expr $NUM / 1048576`

B=`ldm ls -p | grep DOMAIN | grep -v primary | awk -F"|" '{print $7}' | sed -e 's/mem=//'`
NUM=0
for i in $B
do
	NUM=`expr $NUM + $i`
done
MEM_GUEST=`expr $NUM / 1048576`
NUM_of_REM=`expr \( $MEM_PRI - $MEM_GUEST \) / 4 \* 4`
if [ -f $work_dir/mem_$$ ];then
	for j in `cat $work_dir/mem_$$ | awk '{print $2}'`
	do
		NUM_of_REM=`expr $NUM_of_REM - $j`
	done
fi

echo "Domain Memory(MB)" | awk '{printf"%-1s%-18s%-10s\n","",$1,$2}' > $work_dir/memlist.tmp
echo "----------------- ----------" | awk '{printf"%-1s%-18s%-10s\n","",$1,$2}' >> $work_dir/memlist.tmp
ldm ls -p | grep DOMAIN | sed -e 's/name=//' -e 's/mem=//' | awk -F"|" '{printf"%-1s%-18s%10d\n","",$2,$7/1048576}' >> $work_dir/memlist.tmp
if [ -f $work_dir/mem_$$ ];then
	cat $work_dir/mem_$$ | awk '{printf"%-1s%-18s%10s%-1s%-15s\n","",$1,$2,"",$3}' >> $work_dir/memlist.tmp
fi
echo "----------------------------" | awk '{printf"%-1s%-29s\n","",$1}' >> $work_dir/memlist.tmp
echo "" >> $work_dir/memlist.tmp
if [ $DKIND = "primary" ];then
	eval echo "The amount of memory that can be allocated: $MEM_MIN to $NUM_of_REM \(MB\)" >> $work_dir/memlist.tmp
else
	eval echo "The amount of memory that can be allocated: max $NUM_of_REM MB" >> $work_dir/memlist.tmp
fi
if [ $NUM_of_REM -lt $MEM_MIN ];then
	echo "There is no enough memory resource that can be allocated for the guest domain." >> $work_dir/memlist.tmp
fi
echo "" >> $work_dir/memlist.tmp
}

############################################################################
# Make execute script
############################################################################
exec_sh(){

if [ -z $1 ];then
	DOMW=$DOMAIN
else
	EXE=$conf_dir/$1-${DATE}.sh
	DOMW=$1
	work_out="${work_dir}/${DOMW}"
fi

cat <<EOF >> $EXE
#!/bin/sh
#Creation date:`LC_TIME=C;date`

PATH=$PATH ; export PATH

EOF

	#---- Setting VCC -------------------------#
	if [ -f $work_out/vcc.tmp ];then
		if [ $DOMW = "primary" ];then
			while read line
			do
				DOM_VCC=(`echo $line`)
				echo "ldm add-vconscon port-range=${DOM_VCC[9]}-${DOM_VCC[11]} ${DOM_VCC[2]} primary" >> $EXE
			done < $work_out/vcc.tmp
			echo "svcadm enable $VNTSD" >> $EXE
		else
			while read line
			do
				DOM_VCC=(`echo $line`)
				echo "ldm add-vconscon port-range=${DOM_VCC[9]}-${DOM_VCC[11]} ${DOM_VCC[2]} $DOMW" >> $EXE
			done < $work_out/vcc.tmp
		fi
	fi

	#---- Setting Domain Name -------------------------#
	
	if [ -f $work_out/dom.tmp ];then
		while read line
		do
			DOM_COMM=(`echo $line`)
			if [ ${DOM_COMM[0]} = "Domain" ];then
				echo "ldm add-domain ${DOM_COMM[2]}" >> $EXE
			fi
			if [ ${DOM_COMM[0]} = "Host" ];then
				HOSTID=${DOM_COMM[2]}
				if [ "$HOSTID" != "Set" ];then
					echo "ldm set-domain hostid="$HOSTID" $DOMW" >> $EXE
				fi
			fi
		done < $work_out/dom.tmp
	fi

	#---- Pattern3 Special CPU (for I/O) -------------------#
	if [ -f $work_out/cpu_p3.tmp ];then
		while read line
		do
			CPU_COMM=(`echo $line`)
			if [ $DOMW != "primary" ];then
				echo "if [ -x /usr/xpg4/bin/iigrep ];then" >> $EXE
				echo "  RST=\`ldm ls | /usr/xpg4/bin/grep -E '^primary '|awk '{print \$5}'\`" >> $EXE
				echo "else" >> $EXE
				echo "        RST=\`ldm ls | grep primary | grep \"          \"|awk '{print \$5}'\`" >> $EXE
				echo "fi" >> $EXE
				echo "CHK=\`expr ${CPU_COMM[3]} \* 2\`" >> $EXE
				echo "if [ \$RST != \$CHK ];then" >> $EXE
				echo "  ldm set-core ${CPU_COMM[3]} primary" >> $EXE
				echo "fi" >> $EXE
			fi
			echo "ldm set-socket socket_id=${CPU_COMM[2]} $DOMW" >> $EXE
		done < $work_out/cpu_p3.tmp
	fi

	#---- Setting CPU ---------------------------------#

	if [ -f $work_out/cpu.tmp ];then
		while read line
		do
			CPU_COMM=(`echo $line`)
			echo "ldm set-core ${CPU_COMM[2]} $DOMW" >> $EXE
			if [ "${CPU_COMM[3]}" = "enabled" ];then
				echo "ldm set-domain max-cores=${CPU_COMM[2]} $DOMW" >> $EXE
			fi
		done < $work_out/cpu.tmp
	fi

	#---- primary only --------------------------------#
	if [ $DOMW = "primary" ];then
		echo "ldm start-reconf $DOMW" >> $EXE
	fi

	#---- Pattern3 Special CPU (for primary) ------------------#
	if [ -f $work_out/cpu_p3p.tmp ];then
		while read line
		do
			CPU_COMM=(`echo $line`)
			echo "ldm set-socket socket_id=${CPU_COMM[2]} $DOMW" >> $EXE
		done < $work_out/cpu_p3p.tmp
	fi

	#---- Setting MEM ---------------------------------#

	if [ -f $work_out/mem.tmp ];then
		while read line
		do
			MEM_COMM=(`echo $line`)
			echo "ldm set-memory ${MEM_COMM[2]}M $DOMW" >> $EXE
		done < $work_out/mem.tmp
	fi

	if [ -f $work_out/mem_p3.tmp ];then
		while read line
		do
			echo "$line" >> $EXE
		done < $work_out/mem_p3.tmp
	fi

	#---- Remove I/O -------------------------------------#

	if [ -f $work_out/remove_io.tmp ];then
		while read line
		do
			RIO_COMM=(`echo $line`)
			echo "ldm remove-io ${RIO_COMM[2]} $DOMW" >> $EXE
		done < $work_out/remove_io.tmp
	fi

	#---- Add I/O -------------------------------------#

	if [ -f $work_out/add_io.tmp ];then
		while read line
		do
			AIO_COMM=(`echo $line`)
			echo "ldm add-io ${AIO_COMM[3]} $DOMW" >> $EXE
		done < $work_out/add_io.tmp
	fi

	#---- Setting VSW -------------------------#
	if [ -f $work_out/vsw.tmp ];then
		while read line
		do
			VSW_COMM=(`echo $line`)
			if [ "${VSW_COMM[4]}" = "No" ];then
				echo "ldm add-vswitch ${VSW_COMM[2]} $DOMW" >> $EXE
			else
				echo "ldm add-vswitch net-dev=${VSW_COMM[4]} ${VSW_COMM[2]} $DOMW" >> $EXE
			fi
		done < $work_out/vsw.tmp
	fi

	#---- Add VNET -------------------------------------#

	if [ -f $work_out/vnet.tmp ];then
		while read line
		do
			ADD_VNET=(`echo $line`)
			echo "ldm add-vnet ${ADD_VNET[2]} ${ADD_VNET[3]} $DOMW" >> $EXE
		done < $work_out/vnet.tmp
	fi

		#---- Create VNET ---------------------------------#

		if [ -f $work_out/vnt.tmp ];then
			while read line
			do
				VNET_COMM=(`echo $line`)
				echo $line | grep "linkprop=phys-state" > /dev/null 2>&1
				if [ $? = 0 ];then
					if [ "${VNET_COMM[4]}" = PVID ];then
						echo "ldm add-vnet pvid=${VNET_COMM[5]} linkprop=phys-state ${VNET_COMM[2]} ${VNET_COMM[3]} $DOMW" >> $EXE
					else
						if [ "${VNET_COMM[4]}" = VID ];then
							echo "ldm add-vnet vid=${VNET_COMM[5]} linkprop=phys-state ${VNET_COMM[2]} ${VNET_COMM[3]} $DOMW" >> $EXE 
						else
							echo "ldm add-vnet linkprop=phys-state ${VNET_COMM[2]} ${VNET_COMM[3]} $DOMW" >> $EXE
						fi
					fi
				else
					if [ "${VNET_COMM[4]}" = PVID ];then
						echo "ldm add-vnet pvid=${VNET_COMM[5]} ${VNET_COMM[2]} ${VNET_COMM[3]} $DOMW" >> $EXE
					else
						if [ "${VNET_COMM[4]}" = VID ];then
							echo "ldm add-vnet vid=${VNET_COMM[5]} ${VNET_COMM[2]} ${VNET_COMM[3]} $DOMW" >> $EXE 
						else
							echo "ldm add-vnet ${VNET_COMM[2]} ${VNET_COMM[3]} $DOMW" >> $EXE
						fi
					fi
				fi	
			done < $work_out/vnt.tmp
		fi

	#---- Setting VC ----------------------------------#

	if [ -f $work_out/vc.tmp ];then
		while read line
		do
			VC_COMM=(`echo $line`)
			if [ "${VC_COMM[3]}" = "" ];then
				echo "ldm set-vconsole service=${VC_COMM[2]} $DOMW" >> $EXE
			else
				echo "ldm set-vconsole port=${VC_COMM[3]} service=${VC_COMM[2]} $DOMW" >> $EXE
			fi
		done < $work_out/vc.tmp
	fi

	#---- Creating VDISK -----------------#

	if [ -f $work_out/vds.tmp ];then
		while read line
		do
			VDS_COMM=(`echo $line`)
			echo "ldm add-vdiskserver ${VDS_COMM[3]} $DOMW" >> $EXE
		done < $work_out/vds.tmp
	fi

	#---- Creating VDISK for OS Image -----------------#

	if [ -f $work_out/os.tmp ];then
		while read line
		do
			OS_COMM=(`echo $line`)
			if [ "${OS_COMM[1]}" = "Local" ];then
				ldm list -o disk -p | grep VDS | grep primary-vds0 > /dev/null 2>&1
				if [ $? != 0 ];then
					LDM_CONF=`ldm ls-config|grep current|awk -F" " '{print $1}'`
					echo "ldm add-vdiskserver primary-vds0 primary" >> $EXE
					echo "ldm rm-config $LDM_CONF" >> $EXE
					echo "ldm add-config $LDM_CONF" >> $EXE
				fi
				echo "ldm add-vdiskserverdevice options=ro "${OS_COMM[5]}" $DOMW-os@primary-vds0" >> $EXE
				echo "ldm add-vdisk cdrom $DOMW-os@primary-vds0 $DOMW" >> $EXE
			fi
		done < $work_out/os.tmp
	fi

	#---- Creating VDISK for guest ------------------------------#

	if [ -f $work_out/gvds.tmp ];then
		while read line
		do
			VDISK_COMM=(`echo $line`)
			if [ "${VDISK_COMM[5]}" = "options=(none)" ];then
				echo "ldm add-vdiskserverdevice ${VDISK_COMM[6]} "${VDISK_COMM[3]}"@"${VDISK_COMM[4]}"" >> $EXE
			else
				echo "ldm add-vdiskserverdevice ${VDISK_COMM[5]} ${VDISK_COMM[6]} "${VDISK_COMM[3]}"@"${VDISK_COMM[4]}"" >> $EXE
			fi
			echo "ldm add-vdisk ${VDISK_COMM[2]} "${VDISK_COMM[3]}"@"${VDISK_COMM[4]}" $DOMW" >> $EXE
		done < $work_out/gvds.tmp
	fi

	#---- Setting OBP ---------------------------------#

	if [ -f $work_out/var.tmp ];then
		while read line
		do
			OBP_COMM=(`echo $line`)
			echo "ldm set-variable ${OBP_COMM[2]} $DOMW" >> $EXE
		done < $work_out/var.tmp
	fi

	#---- Adding Config file --------------------------#
	if [ -f $work_out/config.tmp ];then
		while read line
		do
			ADD_CONF=(`echo $line`)
			if [ "${ADD_CONF[5]}" = "(new" -o $DOMW = "primary" ];then
				echo "ldm add-config ${ADD_CONF[4]}" >> $EXE
			else
				echo "ldm rm-config ${ADD_CONF[4]}" >> $EXE
				echo "ldm add-config ${ADD_CONF[4]}" >> $EXE
			fi
		done < $work_out/config.tmp
	else
		LDM_CONF=`ldm ls-config|grep current|awk -F" " '{print $1}'`
		if [ "$LDM_CONF" = "" ];then
			LDM_CONF=`ldm ls-config | grep ] |awk -F" " '{print $1}'`
		fi
		echo "ldm rm-config ${LDM_CONF}" >> $EXE
		echo "ldm add-config ${LDM_CONF}" >> $EXE
	fi

	#--------------------------------------------------#

	echo "The configuration file ($EXE) was saved."
	echo ""
	chmod +x $EXE 

}

############################################################################
# Get Start Memory Address and Max memory size
# P1: Address
# P2: grep param
############################################################################
get_mem_addr()
{
	#--- Max size ---#
	A=`ldm list-devices -p -a mem | grep $2 | grep $1`
	B=0
	C=0
	for i in $A
	do
		sz=`echo $i|awk -F"|" '{print $3}' | sed -e 's/size=//'`
		if [ $sz -gt $B ];then
			B=$sz
			C=`echo $i|awk -F"|" '{print $2}' | sed -e 's/pa=//'`
		fi
	done
	NUM_of_REM=`expr $B / 1048576`

	MCHK=`expr $NUM_of_REM % 256`
	if [ "$MCHK" != 0 ];then
		NUM_of_REM="`expr \( $NUM_of_REM / 256 - 1 \) \* 256`"		
	fi

	#--- Start Address ---#
	PA16=`echo $C | sed -e 's/0x//' | tr "[a-z]" "[A-Z]"`
	PA10=`echo "obase=10; ibase=16; $PA16" | bc`
	PACHK=`expr $PA10 % 268435456`
	if [ $PACHK = 0 ];then
		START_MEM="`echo 0x${PA16} | tr "[A-Z]" "[a-z]"`"
	else
		c=`expr \( $PA10 / 268435456 \) \* 268435456 + 268435456 `
		PA16=`echo "obase=16; ibase=10; $c" | bc | tr "[A-Z]" "[a-z]"`
		START_MEM="0x$PA16"
	fi

}
############################################################################
# Assign CPU
# P1: nunber of title
# P2: set max-core(0:none 1:use)
# P3: count
# P4: max-core
############################################################################
domain_cpu(){

if [ "$NUM_of_REM" -le 0 ];then
	echo "This operation is abort. Please press the enter key."
	read key
	delete
	menu
fi

while :
do
	if [ $DKIND = "primary" ];then
		echo "Enter the quantity of CPU cores for the $DKIND domain. [$CPU_DEF] ==> \c"
	else
		echo "Enter the quantity of CPU cores for the $DKIND domain $DOMAIN. [$CPU_DEF] ==> \c"
	fi
	read CORE
	case ${CORE:-$CPU_DEF} in
	  [t])	top
		;;
	  [q])	quit
		;;
	    *)	expr ${CORE:-$CPU_DEF} + 1 > /dev/null 2>&1
		if [ $? = 0 ];then
			if [ ${CORE:-$CPU_DEF} -ge $CPU_MIN -a ${CORE:-$CPU_DEF} -le $NUM_of_REM ];then
				echo ""
				if [ $DKIND = "primary" ];then
					echo "CPU    : ${CORE:-$CPU_DEF} cores for the $DKIND domain." >> $work_out/cpu.info
				else
					echo "CPU    : ${CORE:-$CPU_DEF} cores for the $DKIND domain $DOMAIN." >> $work_out/cpu.info
				fi
				echo "$DOMAIN ${CORE:-$CPU_DEF} (Reserved)" >> $work_dir/cpu_$$
				break
			else
				echo "ERROR: Invalid input value."
				eval echo "ERROR: Please enter a number between $CPU_MIN and $NUM_of_REM."
			fi
		else
			echo "ERROR: Invalid input value."
			eval echo "ERROR: Please enter a number between $CPU_MIN and $NUM_of_REM."
		fi
		;;
	esac
done

if [ $1 = "1" ];then
	while :
	do
		echo "Do you wish to set the parameter of \"max-cores\"?"
		echo "Enter y(yes) or n(no) [n] ==> \c"
		read YN
		case ${YN:-n} in
		  [t])  top
			continue
			;;
		  [q])  quit
			continue
			;;
		  [y])  echo ""
			echo "CPU    : The parameter of \"max-cores\" enabled." >> $work_out/cpu.info
			echo "CPU    : ${CORE:-$CPU_DEF} enabled" >> $work_out/cpu.tmp
			break
			;;
		  [n])  echo ""
			echo "CPU    : The parameter of \"max-cores\" disabled." >> $work_out/cpu.info
			echo "CPU    : ${CORE:-$CPU_DEF} disabled" >> $work_out/cpu.tmp
			break
			;;
		    *)  echo "ERROR: Invalid input value."
			echo ""
			;;
		esac
	done
else
	echo "CPU    : ${CORE:-$CPU_DEF} disabled" >> $work_out/cpu.tmp
fi

#eval $3 = ${CORE:-$CPU_DEF}

}

############################################################################
# Config file name
# P1: file name(i/o)
############################################################################
config_name()
{
CONFIG=$1
ldm list-config | grep $CONFIG > /dev/null 2>&1
if [ $? = 0 ];then
	echo "The configuration file named $CONFIG already exists."
	echo ""
	echo "---------------------------------------------------------------------------------------"
	while :
	do
		echo "Enter the new configuration file name. ==> \c"
		read NEW_CONF
		case ${NEW_CONF:-""} in
		  [t])	top
			continue
			;;
		  [q])	quit
			continue
			;;
		    *)	if [ ${NEW_CONF:-""} = "" ];then
		 		echo "ERROR: Invalid input value."
				echo ""
				continue
			else
				ldm list-config | grep -w ${NEW_CONF:-""} > /dev/null 2>&1
				if [ $? = 0 -a $CONFIG != ${NEW_CONF:-""} ];then
					echo "ERROR: Configuration file named ${NEW_CONF:-""} already exists."
					echo ""
					continue
				else
					CONFIG=${NEW_CONF:-""}
					break
				fi
			fi
			;;
		esac
	done

fi

#eval $2 = $CONFIG

}

############################################################################
# Save config and execute config 
# P1:EXE make for each domain(0:no 1:yes)
############################################################################
save_exec()
{
REBOT=""
REBOT2="following command."
if [ $DOMAIN = "primary" ];then
	REBOT="Please reboot the system after pressing the enter key."
	REBOT2="following command and then reboot the system."
fi
while :
do
	YN=""
	echo "This tool makes a configuration file that can be used to create a domain with the"
	echo "configuration shown above."
	echo "Save this configuration file? If no, the configuration will be cleared. (y/n) ==> \c"
	read YN
	case $YN in
	  [y])	if [ $1 = "1" ];then
						for i in `ls -l $work_dir|grep drwxr-xr-x|awk -F" " '{print $9}'`
						do
							exec_sh $i
						done
					else
						exec_sh
					fi
		while :
		do
			YN=""
			echo "Do you wish to create the domain now with this configuration file? (y/n) ==> \c"
			read YN
				case $YN in
				  [y])	H=`ls $conf_dir/*-${DATE}.sh`	# */
					for i in $H
					do
						echo "Creating the domain with the configuration file ($i)."
						echo ""
						$i
						echo "---------------------------------------------------------------------------------------"
						echo "If error messages are displayed, please check if creation of the domain failed."
						echo "If no messages are displayed, the domain was created successfully."
						echo "---------------------------------------------------------------------------------------"
					done
					echo $REBOT
					read key
					break 2
					;;
				  [n])	echo "---------------------------------------------------------------------------------------"
					echo "The domain was not created."
					echo "If you wish to create the domain with this configuration file later, please execute the"
					echo $REBOT2
					H=`ls $conf_dir/*-${DATE}.sh`	# */
					for i in $H
					do
						echo "# $i"
					done
					echo "Please press the enter key."
					read key
					break 2
					;;
				    *)	echo "Please enter y(yes) or n(no)."
					continue
					;;
				esac
		done

		break
		;;
	  [n])	echo "The operation was aborted, and the domain configuration information was not saved."
		break
		;;
	    *)	echo "Please enter y(yes) or n(no)."
		;;
	esac
done
}
############################################################################
# Creating Number of Domain 
# P1: input value
############################################################################
domain_no(){

while :
do
	echo "Enter the quantity of guest domains that you wish to create.[1] ==> \c"
	read NUM
	case ${NUM:-1} in
	  [t])	top
		;;
	  [q])	quit
		;;
	    *)	expr ${NUM:-1} + 1 > /dev/null 2>&1
		if [ $? = 0 ];then
			if [ ${NUM:-1} -lt 1 ];then
				echo "ERROR: Invalid input value."
			else
				NUM=${NUM:-1}
				break
			fi
		else
			echo "ERROR: Invalid input value."
		fi
		;;
	esac
done

#eval $1 = $NUM
}

############################################################################
# Creating name of Domain
# P1: input value
############################################################################
domain_name(){
#---get domain name--------------------------------------------------------#
NAME=`ldm ls -p | grep DOMAIN | sed -e 's/name=//' -e 's/ncpu=//' | awk -F"|" '{print $2}'`

while :
do
	echo "Enter a logical domain name (e.g., ldom1) ==> \c"
	read DOMAIN
	case ${DOMAIN:-} in
		[t])	top
			;;
		[q])	quit
			;;
 		 "")	echo "ERROR: Invalid input value."
			echo "ERROR: Please enter domain name."
			echo ""
			;;
		  *)	echo $NAME | grep -w $DOMAIN > /dev/null 2>&1
			if [ $? = 0 ];then
				echo "This name is already used by the other domain."
				echo "Please enter another domain name."
				echo ""
				continue
			else
				if [ -d $work_dir/$DOMAIN ];then
					echo "This name is already used by the other domain."
					echo "Please enter another domain name."
					echo ""
					continue
				else
					mkdir $work_dir/$DOMAIN > /dev/null 2>&1
					break
				fi
			fi
			;;
	esac
done

#eval $1 = $DOMAIN

}

############################################################################
# Setting Host ID
# P1: 
############################################################################
host_id(){

while :
do
	YN=""
	echo "Specify a host ID? (If no, a host ID will be created automatically.)(y/[n]) ==> \c"
	read YN
	case ${YN:-n} in 
		[t])	top
			;;
		[q])	quit
			;;
		[n])	echo "Domain : $DOMAIN" >> $work_out/dom.tmp
			echo "Host ID: Set automatically" >> $work_out/dom.tmp
			HOSTID=none
			break
			;;
		[y])	while :
			do
				echo "Enter an 8 character host ID for $DOMAIN ==> \c"
				read HOSTID
				if [ "$HOSTID" = t ];then
					top
					echo ""
				elif [ "$HOSTID" = q ];then
					quit
					echo ""
				fi

				ID_COUNT=`echo $HOSTID | wc -m`
				if [ "$ID_COUNT" -eq 9 ];then
					echo "Domain : $DOMAIN" >> $work_out/dom.tmp
					echo "Host ID: $HOSTID" >> $work_out/dom.tmp
					break 2
				else
					echo "ERROR: Invalid input value."
				fi
			done
	esac
done

#eval $1 = $HOSTID

}


############################################################################
# Assign memory to the guest domain. 
# P1: nunber of title
############################################################################
domain_memory()
{

if [ "$NUM_of_REM" -lt $MEM_MIN ];then
	echo "This operation is abort. Please press the enter key."
	read key
	delete
	menu
fi

while :
do
	if [ $DKIND = "primary" ];then
		echo "Enter the amount of memory for the $DKIND domain. (By multiple of 4MB) [$MEM_DEF](MB)"
	else
		echo "Enter the amount of memory for the $DKIND domain $DOMAIN. (By multiple of 4MB) [$MEM_DEF](MB)"
	fi
	echo "==> \c"
	read MEM
	expr ${MEM:-$MEM_DEF} + 1 > /dev/null 2>&1
	if [ $? = 0 ];then
		if [ ${MEM:-$MEM_DEF} -ge $MEM_MIN -a ${MEM:-$MEM_DEF} -le $NUM_of_REM ];then
			MCHK=`expr ${MEM:-$MEM_DEF} % 4`
			if [ $MCHK = 0 ];then
				echo ""
				if [ $DKIND = "primary" ];then
					echo "Memory : ${MEM:-$MEM_DEF} MB for the $DKIND domain." > $work_out/mem.tmp
				else
					echo "Memory : ${MEM:-$MEM_DEF} MB for the $DKIND domain $DOMAIN." > $work_out/mem.tmp
				fi
				echo "$DOMAIN ${MEM:-$MEM_DEF} (Reserved)" >> $work_dir/mem_$$
				break
			else
				MEM4="`expr \( $MEM / 4 + 1 \) \* 4`"
				echo ""
				echo "WARNING: A memory capacity for a domain needs to allocate by multiple of 4MB."
				echo "WARNING: Although you set \"${MEM}MB\", \"${MEM4}MB\" was specified to the memory capacity."
				echo ""
				if [ $DKIND = "primary" ];then
					echo "Memory : $MEM4 MB for the $DKIND domain." > $work_out/mem.tmp
				else
					echo "Memory : $MEM4 MB for the $DKIND domain $DOMAIN." > $work_out/mem.tmp
				fi
				echo "$DOMAIN ${MEM4:-$MEM_DEF} (Reserved)" >> $work_dir/mem_$$
				break
			fi
		else
			echo "ERROR: Invalid input value."
			echo "ERROR: Please enter a number between $MEM_MIN and $NUM_of_REM."
			echo ""
		fi
	else
		if [ ${MEM:-$MEM_DEF} = t ];then
			top
		elif [ ${MEM:-$MEM_DEF} = q ];then
			quit
		else
			echo "ERROR: Invalid input value."
			echo "ERROR: Please enter a number between $MEM_MIN and $NUM_of_REM."
			echo ""
		fi
	fi
done

}

############################################################################
# Method of installing Solaris OS
# P1: VDS select(0:no use 1:use)
############################################################################
os_install()
{

VDS_LIST=`ldm list -o disk -p | grep VDS | awk -F"|" '{print $2}' | sed -e s/^name=//`

while :
do
	echo "Please select the installation method."
	echo " 1. Network-based installation"
	echo " 2. Installation from an iso image file"
	echo "Enter a number.[1] ==> \c"
	read METHOD

	case ${METHOD:-1} in
	[t])	top
		;;
	[q])	quit
		;;
	[1])	if [ -e $work_out/vnet.tmp -o -e $work_out/vnt.tmp ];then
			echo ""
		else
			echo ""
			if [ $DKIND = "I/O" ];then
				echo "---------------------------------------------------------------------------------------"
				echo "For network installation, please use the network interface on the I/O device that you"
				echo "assigned to the $DKIND domain."
				echo "---------------------------------------------------------------------------------------"
			else
				echo "For a network installation, you must assign a VNET to the guest domain."
				echo ""
				continue
			fi
		fi
		echo "Install: Network-based installation" >> $work_out/os.info
		break
		;;
	[2])	echo ""
		while :
		do
			if [ -f $work_out/imgpath.tmp ];then
				rm $work_out/imgpath.tmp
			fi
			echo "Enter the Solaris OS ISO image file (with full path)."
			echo "==> \c"
			echo `bash -c 'read -e answer ; echo $answer'` >> $work_out/imgpath.tmp
			IMGPATH=`cat $work_out/imgpath.tmp`
			if [ ${IMGPATH:-" "} = t ];then
				top
			elif [ ${IMGPATH:-" "} = q ];then
				quit
			elif [ ${IMGPATH:-" "} = b ];then
				break
			else
				if [ -f ${IMGPATH:-" "} ];then
					ISOCHK=`file ${IMGPATH:-" "} | awk '{print $2}'`
					if [ "$ISOCHK" = "ISO" ];then
						break
					else
						echo "ERROR: Not an iso image file."
						echo ""
						rm $work_out/imgpath.tmp
						continue
					fi
				else
					echo "ERROR: No such file."
					echo ""
					rm $work_out/imgpath.tmp
					continue
				fi
			fi
		done

		if [ ${IMGPATH:-" " } = b ];then
			continue
		fi

		if [ $1 = "1" ];then
			while :
			do
				echo ""
				echo "Select the virtual disk server that manages the virtual disk with the iso image file"
				echo "from the following list:"
				D=0
				for i in `echo $VDS_LIST`
				do
					D=`expr $D + 1`
					echo " $D. $i"
				done
				echo "Enter a number ==> \c"
				read j

				VDS_WC=`echo "$VDS_LIST" | wc -l`
				expr $j + 1 > /dev/null 2>&1
				if [ $? = 0 ];then
					if [ $j -ge 1 -a $j -le $VDS_WC ];then
						VDS_OS=`echo $VDS_LIST | awk '{print $'$j'}'`
						echo "Install: Local ( Image path $IMGPATH )( VDS $VDS_OS ) " >> $work_out/os.tmp
						echo "Install: Installation from an iso image file." >> $work_out/os.info
						echo "Install: The $IMGPATH image file will be used." >> $work_out/os.info
						echo "Install: $VDS_OS will be used as the virtual disk server." >> $work_out/os.info
						echo ""
						break 2
					else
						echo "ERROR: Invalid input value."
						echo ""
						continue
					fi
				else
					echo "ERROR: Invalid input value."
					echo ""
					continue
				fi
			done
		else
			echo "Install: Local ( Image path $IMGPATH )( VDS primary-vds0 ) " >> $work_out/os.tmp
			echo "Install: Installation from an iso image file." >> $work_out/os.info
			echo "Install: The $IMGPATH image file will be used." >> $work_out/os.info
			echo "Install: primary-vds0 will be used as the virtual disk server." >> $work_out/os.info
			echo ""
		fi
		break
		;;
	*)	echo ""
		continue
		;;
	esac
done

}

############################################################################
# Add VSW
############################################################################
add_vsw()
{
if ! [ -e $work_dir/showphys.tmp_$$ ];then
	dladm show-phys | grep Ethernet | sed -n 1,\$p | sort | sed -n /^net/p | sed -e s/^net// | sort -n | sed -e s/^/net/ >> $work_dir/showphys.tmp_$$
	dladm show-phys | grep Ethernet | sed -n 1,\$p | sort | sed -e /^net/d >> $work_dir/showphys.tmp_$$
	#dladm show-link | grep aggr | awk -F" " '{print $1}' > $work_dir/aggrnic.tmp_$$
	dladm show-link | grep aggr > $work_dir/aggr.tmp_$$
	if [ -f $work_dir/aggr.nic ];then
		rm $work_dir/aggr.nic
	fi
	while read line
	do
	  IO_NO=4
	  DOM=(`echo $line`)
		printf "%s %s %s %s unknown " "${DOM[0]}" "${DOM[1]}" "${DOM[2]}" "${DOM[3]}" >> $work_dir/showphys.tmp_$$
	  while :
	  do
	    if [ ${DOM[IO_NO]} ]; then
	      printf "%s_" "${DOM[IO_NO]}" >> $work_dir/showphys.tmp_$$
				echo "${DOM[IO_NO]}" >> $work_dir/aggr.nic
	      IO_NO=`expr $IO_NO + 1`
	    else
				printf "\n" >> $work_dir/showphys.tmp_$$
	      break
	    fi
	  done
	done < $work_dir/aggr.tmp_$$
fi

echo ""
echo "This virtual switch service is named $DOMAIN-vsw$1."
echo "Please select a physical NIC to allocate $DOMAIN-vsw$1 from the following list."
COUNT03=1
while read line
do
	PHYS=(`echo $line`)
	addm=""
	if [ -f $work_dir/aggr.nic ];then
		while read aggr
		do
			if [ ${PHYS[0]} = $aggr ];then
				addm=" This NIC has been used for Link Aggregation."
				break
			fi
		done < $work_dir/aggr.nic
	fi
	if [ "${PHYS[6]}" = "" ];then
		if [ "${PHYS[1]}" = "Ethernet" ];then
			echo " $COUNT03. ${PHYS[0]} (physical: ${PHYS[5]} )" $addm
		else
			echo " $COUNT03. ${PHYS[0]} (Link Aggregation )"
		fi
	else
		if [ "${PHYS[1]}" = "Ethernet" ];then
			echo " $COUNT03. ${PHYS[0]} (physical: ${PHYS[5]} )(allocated to the ${PHYS[6]})"
		else
			echo " $COUNT03. ${PHYS[0]} (Link Aggregation )(allocated to the ${PHYS[6]})"
		fi
	fi
	COUNT03=`expr $COUNT03 + 1`
done < $work_dir/showphys.tmp_$$
echo " $COUNT03. Do not assign a physical NIC"

while :
do
	echo "Enter a number ==> \c"
	read NUM
	if [ "$NUM" = "" ];then
		echo "ERROR: Invalid input value."
		echo "ERROR: Please enter a number between 1 and $COUNT03."
		echo "Please press the enter key."
		read
		continue
	fi
	expr $NUM + 1 > /dev/null 2>&1
	if [ $? = 0 ];then
		if [ $NUM -ge 1 -a $NUM -le $COUNT03 ];then
			if [ "$NUM" = $COUNT03 ];then
				echo "VSW    : $DOMAIN-vsw$1 : No physical NIC assigned." >> $work_out/vsw.tmp
				echo "VSW    : $DOMAIN-vsw$1 : No physical NIC assigned." >> $work_out/vsw.info
			else
				PNIC=(`sed -n "$NUM"p $work_dir/showphys.tmp_$$`)
				ERROR=0
				if [ -f $work_dir/aggr.nic ];then
					while read aggr
					do
						if [ ${PNIC[0]} = $aggr ];then
							ERROR=1
							break
						fi
					done < $work_dir/aggr.nic
					if [ $ERROR = 1 ];then
						echo "ERROR: The selected NIC has already been used for Link Aggregation."
						echo "ERROR: Please select another NIC." 
						echo "Please press the enter key."
						read key
						continue
					fi
				fi
				if [ "${PNIC[6]}" = "" ];then
					echo "VSW    : $DOMAIN-vsw$1 : ${PNIC[0]} was allocated." >> $work_out/vsw.tmp
					if [ "${PNIC[1]}" = "Ethernet" ];then
						echo "VSW    : $DOMAIN-vsw$1 : ${PNIC[0]} [${PNIC[5]}] was allocated." >> $work_out/vsw.info
					else
						echo "VSW    : $DOMAIN-vsw$1 : ${PNIC[0]} [Link Aggregation] was allocated." >> $work_out/vsw.info
					fi
				else
					echo "ERROR: The selected NIC has already been used."
					echo "ERROR: Please select another NIC." 
					echo "Please press the enter key."
					read key
					continue
				fi
				LINE=`sed -n "$NUM"p $work_dir/showphys.tmp_$$`
				sed -e s/"$LINE"/"$LINE $DOMAIN-vsw$1"/ $work_dir/showphys.tmp_$$ >>  $work_dir/showphys.tmp2_$$
				mv  $work_dir/showphys.tmp2_$$ $work_dir/showphys.tmp_$$ > /dev/null 2>&1
			fi
			break
		else
			echo "ERROR: Invalid input value."
			echo "ERROR: Please enter a number between 1 and $COUNT03."
			echo "Please press the enter key."
			read
			continue
		fi
	else
		if [ $NUM ];then
			if [ $NUM = t ];then
				top
			elif [ $NUM = q ];then
				quit
			else
				echo "ERROR: Invalid input value."
				echo "ERROR: Please enter a number between 1 and $COUNT03."
				echo "Please press the enter key."
				read
				continue
			fi
		else
			echo "ERROR: Invalid input value."
			echo "ERROR: Please enter a number between 1 and $COUNT03."
			echo "Please press the enter key."
			read
			continue
		fi
	fi
done
}

############################################################################
# Add Vcc
############################################################################
add_vcc()
{
while :
do
	YN=""
	echo "Do you wish to specify a virtual console port number of virtual console for $DOMAIN?"
	echo "(y/[n]) ==> \c"
	read YN 
	case ${YN:-n} in
		[t])	top
			;;
		[q])	quit
			;;
               [yn])	break
			;;
		  *)	continue
			;;	
	esac
done

RANGE=`ldm ls -l -p | grep VCC | grep $VCC | awk -F"|" '{print $4}'`
RANGE_MIN=`echo $RANGE | sed -e s/port-range=// | awk -F"-" '{print $1}'`
RANGE_MAX=`echo $RANGE | sed -e s/port-range=// | awk -F"-" '{print $2}'`

if [ ${YN:-n} = y ];then
	while :
	do
		echo "Enter a virtual console port number. (port range:"$RANGE_MIN"-"$RANGE_MAX") ==> \c"
		read VCPORT
			case ${VCPORT:-n} in
			  [t])	top
				;;
			  [q])	quit
				;;
			  [n])	echo "ERROR: Invalid input value."
				;;
			    *)	expr ${VCPORT:-n} + 1 > /dev/null 2>&1
				if [ $? = 0 ];then
					if [ $VCPORT -ge $RANGE_MIN -a $VCPORT -le $RANGE_MAX ];then
						echo "VC     : $VCC $VCPORT" >> $work_out/vc.tmp
						echo "VC     : the virtual console concentrator is $VCC" >> $work_out/vc.info
						echo "VC     : the port number is $VCPORT" >> $work_out/vc.info
						break
					else
						echo "ERROR: Invalid input value."
						echo ""
					fi
				else
					echo "ERROR: Invalid input value."
					echo ""
				fi
				;;
		esac
	done
else
	echo "VC     : $VCC" >> $work_out/vc.tmp
	echo "VC     : the virtual console concentrator is $VCC" >> $work_out/vc.info
fi
}

############################################################################
# Set Variables
############################################################################
set_var()
{
while :
do
	echo "Select the "auto-boot?" parameter value for $DOMAIN."
	echo " 1. auto-boot=false"
	echo " 2. auto-boot=true (OS boots automatically with starting a $DKIND domain)"
	echo "Enter a number [1] ==> \c"
	read i

	case ${i:-1} in
		[t])	top
			continue
			;;
		[q])	quit
			continue
			;;
		[1])	echo "OBP    : $DOMAIN: auto-boot?=false" >> $work_out/var.info
					echo "OBP    : auto-boot?=false" >> $work_out/var.tmp
			break
			;;
		[2])	echo "OBP    : $DOMAIN: auto-boot?=true" >> $work_out/var.info
					echo "OBP    : auto-boot?=true" >> $work_out/var.tmp
			break
			;;
		  *)	echo "ERROR: Invalid input value."
			echo ""
			;;
	esac
done

while :
do
	echo ""
	echo "Enter "boot-device" parameter value (the device name from which to boot) for $DOMAIN."
	echo "(e.g., vdisk0) ==> \c"
	read BOOTDEV
	case ${BOOTDEV:-} in
		[t])	top
			continue
			;;
		[q])	quit
			continue
			;;
	 	 "")	break
			;;
		  *)	echo "OBP    : $DOMAIN: boot-device=$BOOTDEV" >> $work_out/var.info
					echo "OBP    : boot-device=$BOOTDEV" >> $work_out/var.tmp
			break
			;;
	esac	
done

}
############################################################################
# Get BackEnd device
############################################################################
get_bke()
{

devfsadm -C

if [ -f $work_dir/bke0.tmp ];then
	return
fi

VDS_LIST2=`ldm list -o disk -p | grep VDS | awk -F"|" '{print $2}' | sed -e s/^name=//`
ZVOL_LIST=`zfs list -t volume | sed -e '1d' | grep -v dump | grep -v swap | awk '{print $1}'`
BKE_LIST=`echo $ZVOL_LIST`
SYSVOL=`zpool status rpool | grep ONLINE | grep s0 | awk '{print $1}'`

b00=0
for e in `echo $ZVOL_LIST`
do
	b00=`expr $b00 + 1`
	echo " $b00. /dev/zvol/dsk/"$e" (ZVOL)" >> $work_dir/bke0.tmp
done

DS_PATH="/dev/dsk/*s2"
for f in `echo $DS_PATH`
do
	DEV=`echo $f | sed -e 's@^/dev/dsk/@@' | sed -e 's@s2$@s0@'`
	echo $SYSVOL | grep $DEV > /dev/null 2>&1
	if [ $? = 0 ];then
		b00=`expr $b00 + 1`
		echo " $b00. $f (DISK) [primary:boot-device]" >> $work_dir/bke0.tmp
	else
		b00=`expr $b00 + 1`
		echo " $b00. $f (DISK)" >> $work_dir/bke0.tmp
	fi
done

#VOLLIST=`ldm ls -l -p | grep VDISK | awk -F"|" '{print $3}' | sed -e s/vol=// | awk -F"@" '{print $1}'`
VOLLIST=`ldm ls -l -p | grep VDISK | awk -F"|" '{print $3}' | sed -e s/vol=//`

while read line
do
	FL1=""
	for i in $VOLLIST
	do
		NAME=`echo $i | awk -F"-" '{print $1}'`
		VOLDEV=`ldm ls -l -p primary | grep $i | awk -F"|" '{print $4}' | sed -e s/dev=//`
		echo "$line" | grep $VOLDEV >> /dev/null 2>&1
		if [ $? = 0 ];then
			echo " $line [$NAME:Reserved]" >> $work_dir/bke.tmp
			FL1=ok
		fi
	done
	if [ "$FL1" = "" ];then
		echo " $line" >> $work_dir/bke.tmp
	fi
done < $work_dir/bke0.tmp

}

############################################################################
# Make vurtual disk for guest domain
# P1: Boot Disk select(0:none 1:enable)
# P2: Option select (0:none 1:enable)
# P3: Selected VDS
############################################################################
make_vdisk()
{

	get_bke

	if [ "$VDS_LIST2" = "" ]; then
		echo "VDS not found.\nCan not create a virtual disk."
		return
	fi
	while :
	do
		echo "Enter the quantity of virtual disks that you wish to create. [1] ==> \c"
		read NUM_of_VDISK
		case $NUM_of_VDISK in
			[t])	top
			;;
			[q])	quit
			;;
			*)	expr ${NUM_of_VDISK:-1} + 1 > /dev/null 2>&1
			if [ $? = 0 ];then
				if [ ${NUM_of_VDISK:-1} -lt $MIN_VDISK ];then
					echo "ERROR: Invalid input value."
					continue
				else
					break
				fi
			else
				echo "ERROR: Invalid input value."
				continue
			fi
			;;
		esac
	done
	
	COUNT0=0
	while [ $COUNT0 -lt ${NUM_of_VDISK:-1} ]
	do
		echo ""
		BD=""
		COUNT1=`expr $COUNT0 + 1`
		case $COUNT1 in
			[1])	echo "=== Set the 1st virtual disk configuration: vdisk$COUNT0 ==="
				if [ $1 = "1" ];then
					echo "------------------------------------------"
					echo "The disk chosen here is set as boot-disk."
					echo "------------------------------------------"
					BD="[boot-disk]"
				fi
				;;
			[2])	echo "=== Set the 2nd virtual disk configuration: vdisk$COUNT0 ==="
				;;
			[3])	echo "=== Set the 3rd virtual disk configuration: vdisk$COUNT0 ==="
				;;
			[4-9])	echo "=== Set the ${COUNT1}th virtual disk configuration: vdisk$COUNT0 ==="
				;;
			*)	echo "=== Set the next virtual disk configuration: vdisk$COUNT0 ==="
				;;
		esac
		k01=0
		if [ $# -eq 3 ];then
			k02=0
			for i in `echo $VDS_LIST2`
			do
				k01=`expr $k01 + 1`
				if [ $i = $3 ];then
					k02=1
					break
				fi
			done
			if [ $k02 = 0 ];then
				echo "VDS not found.\nCan not create a virtual disk."
				return
			fi
			i=$k01
		else
			echo "Please select a virtual disk server for vdisk$COUNT0 from the following list:"
			for i in `echo $VDS_LIST2`
			do
				k01=`expr $k01 + 1`
				echo " ${k01}. $i"
			done
			if [ $k01 -eq 0 ];then
				echo "Can not create a virtual disk."
				return
			fi
			if [ $k01 -gt 1 ];then
				echo "Enter a number ==> \c"
				read i
				case $i in
					[t])	top
						;;
					[q])	quit
						;;
					*)	expr $i + 1 > /dev/null 2>&1
						if [ $? = 0 ];then
							if [ $i -gt $k01 -o $i -lt 1 ];then
								echo "ERROR: Invalid input value."
								continue
							fi
						else
							echo "ERROR: Invalid input value."
							continue	
						fi
						;;
				esac
			else
				i=1
			fi
		fi
		VDS_T=`echo $VDS_LIST2 | awk '{print $'$i'}'`

		echo $VDS_T | grep primary > /dev/null 2>&1
		if [ $? = 0 ];then
			echo ""
			print "---------------------------------------------------------------------------------------"
			print "The virtual disk back end is where virtual disk data is stored. The virtual disk back "
			print "end can be a physical disk, a disk slice, a file, or a volume (such as ZFS, Solaris "
			print "Volume Manager, or VxVM volume)."
			printf "Select the virtual disk back end device for vdisk$COUNT0 from the following or directly\n" "$COUNT0"
			print "enter a device path."

			while :
			do
				echo "---------------------------------------------------------------------------------------"
				cat $work_dir/bke.tmp
				echo "---------------------------------------------------------------------------------------"
				echo "This tool cannot recognize the availability of all devices in all cases. "
				echo "For all devices above that are not marked as reserved or in use as boot-devices,"
				echo "please confirm that the device is not in use by the primary domain."
				echo "For example, confirm that the primary domain is not using the device in a zfs pool."
				echo "---------------------------------------------------------------------------------------"
				echo "Enter a number (or enter a device path) ==> \c"

				if [ -f $work_dir/imgpath.tmp ];then
					rm $work_dir/imgpath.tmp
				fi

				echo `bash -c 'read -e answer ; echo $answer'` >> $work_dir/imgpath.tmp
				i=`cat $work_dir/imgpath.tmp`
				if [ ${i:-" "} = t ];then
					top
				elif [ ${i:-" "} = q ];then
					quit
				else
					expr ${i:-" "} + 1 > /dev/null 2>&1
					if [ $? = 0 ];then
						cat $work_dir/bke.tmp | sed -n "$i"p | awk '{print $4}' | grep ":Reserved\]" > /dev/null 2>&1 #ver1.1
						if [ $? = 0 ];then
							echo "---------------------------------------------------------------------------------------"
							echo "Selected device has already been selected." # ver1.1
							echo "Please select another device. Please press the enter key."
							read key
							continue
						fi

						BKE_T=`cat $work_dir/bke.tmp | sed -n "$i"p | awk '{print $2,$3}'`
						cat $work_dir/bke.tmp | sed -n "$i"p | grep "primary:boot-device" > /dev/null 2>&1
						if [ $? = 0 ];then
							echo "---------------------------------------------------------------------------------------"
							echo "Selected device is using by the boot device of the primary domain."
							echo "Please select another device. Please press the enter key."
							read key
							continue
						else
							if [ $i -lt 1 -o $i -gt $b00 ];then
								echo "---------------------------------------------------------------------------------------"
								echo "ERROR: Invalid input value."
								read key
								continue
							fi
						fi

						bke_count=1
						while read line
						do
							if [ $bke_count = $i ];then
								echo " $line [$DOMAIN:Reserved]" >> $work_dir/bke_$$
							else
								echo " $line" >> $work_dir/bke_$$
							fi
							bke_count=`expr $bke_count + 1`
						done < $work_dir/bke.tmp
						rm $work_dir/bke.tmp
						cp $work_dir/bke_$$ $work_dir/bke.tmp
						rm $work_dir/bke_$$
						break
					else
						if [ -f ${i:-" "} -o -b ${i:-" "} -o -c ${i:-" "} ];then
							BKE_T=$i
							break
						else
							echo "---------------------------------------------------------------------------------------"
							echo "No such file. Please press the enter key."
							read key
							continue
						fi
					fi	
				fi
			done
		else
			echo "Enter a full path of backend device for vdisk$COUNT0. ==> \c"
			read BKE_T
			case ${BKE_T:-} in
				[t])	top
					;;
				[q])	quit
					;;
		 		 "")	echo "ERROR: Invalid input value."
					echo "ERROR: Please enter a full path of backend device for vdisk$COUNT0."
					echo ""
					continue
					;;
				  *)	echo $BKE_T > /dev/null 2>&1
					;;
			esac
		fi
		if [ $2 = "1" ];then
			while :
			do
				echo ""
				echo "Please select an option for vdisk$COUNT0 from the following list:"
				echo " 1. ro (access read-only)"
				echo " 2. excl (access exclusively)"
	#			echo " 3. slice (for mounting an iso file)"
				echo " n. none"
				echo "Enter a number [n] ==> \c"
				read OPTION
					case ${OPTION:-n} in
					  [n])	VDISK_OPTION="options=\(none\)"
						P_OPTION="no option"
						break
						;;
					  [1])	VDISK_OPTION="options=ro"
						P_OPTION="ro option added"
						break
						;;
					  [2])	VDISK_OPTION="options=excl"
						P_OPTION="excl option added"
						break
						;;
	#				  [3])	VDISK_OPTION="options=slice"
	#					P_OPTION="slice option added"
	#					break
	#					;;
					    *)	continue
						;;
					esac
			done
		else
			VDISK_OPTION="options=\(none\)"
			P_OPTION="no option"
		fi
		BKE_W=`echo $BKE_T | awk '{print $1}'`
		echo "VDISK  : vdisk$COUNT0 $DOMAIN-vol$COUNT0 $VDS_T $VDISK_OPTION $BKE_W" >> $work_out/gvds.tmp
		echo "VDISK  : vdisk$COUNT0: the service name is $VDS_T" >> $work_out/gvds.info
		echo "VDISK  : vdisk$COUNT0: the volume name is $DOMAIN-vol$COUNT0" >> $work_out/gvds.info
		echo "VDISK  : vdisk$COUNT0: $P_OPTION" >> $work_out/gvds.info
		echo "VDISK  : vdisk$COUNT0: the disk file is $BKE_T $BD" >> $work_out/gvds.info
		
		COUNT0=`expr $COUNT0 + 1`
	done

}
