#/bin/bash
#
# All Rights Reserved, Copyright (c) FUJITSU LIMITED 2012-2014
#
#	Configuration Tool for Oracle VM Server for SPARC
#
# Revision 2.2  2014/10/02
#
# pattern4_primary.sh

primary_pattern4()
{
DOMAIN=primary

#---set default------------------------------------------------------------#
CPU=4
CPU_DEF=1
MEM=16384
MEM_DEF=16384
STPORT="5000"
EDPORT="5100"
NET="net1"
NOWD=`date '+%y%m%d'`
CONFIG="config-${NOWD}"
PRIIO="pci@8000 pci@8100"
if [ "$MODEL" = "T4-2" -o "$MODEL" = "T4-4" ];then
	PRIIO="pci@400"
fi
#--------------------------------------------------------------------------#
TITLE=" 1.Configure a Primary Domain"
DKIND="primary"
PATTERN="Pattern 4"
work_out="${work_dir}"

############################################################################
# Assign CPU
############################################################################
	echo "CPU    : ${CPU:-$CPU_DEF} disabled" >> $work_out/cpu.tmp

############################################################################
# Setting Memory
############################################################################
	echo "Memory : ${MEM:-$MEM_DEF} MB for the primary domain $DOMAIN" >> $work_out/mem.tmp

############################################################################
# Confirm removed I/O devices
############################################################################

#--- get I/O device -------------------------------------------------------#
N=0
if [ "$MODEL" = "M10-1" -o "$MODEL" = "M10-4" -o "$MODEL" = "M10-4S" ];then
	ldm list-io -p | grep BUS | sed -e 's/|/ /g' | awk '{print $1,$5,$4}' | sed -e 's/dev=//' | sed -e 's/domain=//' | sed -e 's/type=//' >> $work_dir/list-io-p.tmp
	while read line
	do
		echo $line | awk '{printf"%-1s%-4s%-16s%-6s%-10s\n","",'$N',$1,$2,$3}' >> $work_dir/iolist.tmp
		N=`expr $N + 1`
	done < $work_dir/list-io-p.tmp
elif [ "$MODEL" = "T4-2" -o "$MODEL" = "T4-4" ];then
	ldm list-io -p | grep "type=BUS" | awk -F"|" '{print $2}' | grep -v pci@400 | sed -e 's/dev=//' >> $work_dir/list-io-p.tmp
	while read line
	do
		echo " $N   $line         BUS   [primary domain]" >> $work_dir/iolist.tmp
		N=`expr $N + 1`
	done < $work_dir/list-io-p.tmp
fi

IODEV=`cat $work_dir/iolist.tmp | awk '{print $2}'`
for i in $PRIIO
do
	IODEV=`echo $IODEV | sed -e 's/'$i'//'`
done

for i in $IODEV
do
	echo "I/O    : $i removed from the primary domain" >> $work_out/remove_io.tmp
done

############################################################################
# get physical network device
############################################################################

COUNT=0
for i in $NET
do
	PHYS="`dladm show-phys | grep -w $i | awk '{print $6}'`"
	echo "VSW    : primary-vsw$COUNT : $i was allocated." >> $work_out/vsw.tmp
	echo "VSW    : primary-vsw$COUNT : $i [$PHYS] was allocated." >> $work_out/vsw.info
	COUNT=`expr $COUNT + 1`
done

############################################################################
# Setting VCC
############################################################################

echo "VCC    : primary-vcc0 will use the port range of $STPORT to $EDPORT" >> $work_out/vcc.tmp

############################################################################
# Check configuration file name
############################################################################

header "$TITLE" "$PATTERN"
CONF_O=$CONFIG
config_name $CONF_O

echo "The configuration name : $CONFIG" >> $work_out/config.tmp

############################################################################
# Confirmation of setting
############################################################################
header "$TITLE" "$PATTERN"
print "The following primary domain configuration will be set."
print ""
print "======= Your configuration information for primary domain ============================="
printf "Domain : %s\n" "$DOMAIN"
printf "CPU    : %s cores for the primary domain\n" "$CPU"
printf "Memory : %s MB for the primary domain\n" "$MEM"
print "-------------------------------------------------------------------"
cat $work_out/remove_io.tmp
print "-------------------------------------------------------------------"
cat $work_out/vcc.tmp
print "-------------------------------------------------------------------"
cat $work_out/vsw.info
print "-------------------------------------------------------------------"
cat $work_out/config.tmp
print "======================================================================================="
print ""

############################################################################
# Execution and creating of configuration file
############################################################################

EXE=$conf_dir/${DOMAIN}-${DATE}.sh

save_exec 0

############################################################################
# Delete temporary file 
############################################################################

delete
############################################################################

}
